#include "ContextToNwdafApi.hpp"

#include <nlohmann/json.hpp>


namespace oai {
namespace udm {
namespace api {


ContextToNwdafApi::ContextToNwdafApi(
    std::shared_ptr<Pistache::Rest::Router> rtr) {
  router = rtr;
}

void ContextToNwdafApi::init() { setupRoutes(); }

void ContextToNwdafApi::setupRoutes() {
  using namespace Pistache::Rest;

  Routes::Get(
      *router, base + "/udm-data",
      Routes::bind(&ContextToNwdafApi::get_context_to_nwdaf_handler, this));

  // Default handler, called when a route is not found
  router->addCustomHandler(Routes::bind(
      &ContextToNwdafApi::context_to_nwdaf_api_default_handler, this));
}

void ContextToNwdafApi::get_context_to_nwdaf_handler(
    const Pistache::Rest::Request &request,
    Pistache::Http::ResponseWriter response) {

  try {
    this->get_context_to_nwdaf(response);
  } catch (nlohmann::detail::exception &e) {
    // send a 400 error
    response.send(Pistache::Http::Code::Bad_Request, e.what());
    return;
  } catch (std::exception &e) {
    // send a 500 error
    response.send(Pistache::Http::Code::Internal_Server_Error, e.what());
    return;
  }
}

void ContextToNwdafApi::context_to_nwdaf_api_default_handler(
    const Pistache::Rest::Request &, Pistache::Http::ResponseWriter response) {
  response.send(Pistache::Http::Code::Not_Found,
                "The requested method does not exist");
}

} // namespace api
} // namespace udm
} // namespace oai

