/*
 * Licensed to the OpenAirInterface (OAI) Software Alliance under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The OpenAirInterface Software Alliance licenses this file to You under
 * the OAI Public License, Version 1.1  (the "License"); you may not use this
 * file except in compliance with the License. You may obtain a copy of the
 * License at
 *
 *      http://www.openairinterface.org/?page_id=698
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *-------------------------------------------------------------------------------
 * For more information about the OpenAirInterface (OAI) Software Alliance:
 *      contact@openairinterface.org
 */

/*! file
brief
author Jian Yang
date 2021
email: contact@openairinterface.org
*/

#include "imsi_context.hpp"

namespace context {

imsi_context::imsi_context() {
  m_messageFlag = false;
  m_imsi = "";
  m_imsiIsSet = false;
  m_op = "";
  m_opIsSet = false;
  m_optype = "";
  m_optypeIsSet = false;
  m_k = "";
  m_kIsSet = false;
}
imsi_context::~imsi_context() {}

void to_json(nlohmann::json& j, const imsi_context& o) {
  j["messageflag"] = o.m_messageFlag;
  if (o.imsiIsSet()) j["imsi"] = o.m_imsi;
  if (o.opIsSet()) j["op"] = o.m_op;
  if (o.opTypeIsSet()) j["optype"] = o.m_optype;
  if (o.kIsSet()) j["k"] = o.m_k;
}
void from_json(const nlohmann::json& j, imsi_context& o) {
  j.at("messageflag").get_to(o.m_messageFlag);
  if (j.find("imsi") != j.end()) {
    j.at("imsi").get_to(o.m_imsi);
    o.m_imsiIsSet = true;
  }
  if (j.find("op") != j.end()) {
    j.at("op").get_to(o.m_op);
    o.m_opIsSet = true;
  }
  if (j.find("optype") != j.end()) {
    j.at("optype").get_to(o.m_optype);
    o.m_optypeIsSet = true;
  }
  if (j.find("k") != j.end()) {
    j.at("k").get_to(o.m_k);
    o.m_kIsSet = true;
  }
}

bool imsi_context::getMessageFlag() const { return m_messageFlag; }
void imsi_context::setMessageFlag(bool const& value){ m_messageFlag = value; }


std::string imsi_context::getImsi() const { return m_imsi; }
void imsi_context::setImsi(std::string const& value) {
  m_imsi = value;
  m_imsiIsSet = true;
}
bool imsi_context::imsiIsSet() const { return m_imsiIsSet; }
void imsi_context::unsetImsi() { m_imsiIsSet = false; }

std::string imsi_context::getOp() const { return m_op; }
void imsi_context::setOp(std::string const& value) {
  m_op = value;
  m_opIsSet = true;
}
bool imsi_context::opIsSet() const { return m_opIsSet; }
void imsi_context::unsetOp() { m_opIsSet = false; }

std::string imsi_context::getOpType() const { return m_optype; }
void imsi_context::setOpType(std::string const& value) {
  m_optype = value;
  m_optypeIsSet = true;
}
bool imsi_context::opTypeIsSet() const { return m_optypeIsSet; }
void imsi_context::unsetOpType() { m_optypeIsSet = false; }

std::string imsi_context::getK() const { return m_k; }
void imsi_context::setK(std::string const& value) {
  m_k = value;
  m_kIsSet = true;
}
bool imsi_context::kIsSet() const { return m_kIsSet; }
void imsi_context::unsetK() { m_kIsSet = false; }

}  // namespace nf_msg
